package eu.ibacz.contactmgr.backend;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import org.apache.commons.dbcp.BasicDataSource;
import java.sql.SQLException;
import javax.sql.DataSource;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;


/**
 *
 * @author Petr Adamek
 */
public class ContactServiceImplTest {
    
    private ContactServiceImpl manager;
    private DataSource ds;

    private static DataSource prepareDataSource() throws SQLException {
        BasicDataSource ds = new BasicDataSource();
        //we will use in memory database
        ds.setUrl("jdbc:derby:memory:contactmgr-test;create=true");
        return ds;
    }

    @Before
    public void setUp() throws SQLException {
        ds = prepareDataSource();
        DBUtils.createTables(ds);
        manager = new ContactServiceImpl();
        manager.setDataSource(ds);

    }

    @After
    public void tearDown() throws SQLException {
        DBUtils.dropTables(ds);
    }

    @Test
    public void addContact() {
        Contact contact = newContact("George", "+420 543 438 190"); 
        manager.addContact(contact);

        Long contactId = contact.getId();
        assertNotNull(contactId);
        Contact result = manager.getContact(contactId);
        assertEquals(contact, result);
        assertDeepEquals(contact, result);
    }
    
    @Test
    public void getContact() {
        
        assertNull(manager.getContact(1l));
        
        Contact contact = newContact("George", "+420 543 438 190"); 
        manager.addContact(contact);
        Long contactId = contact.getId();

        Contact result = manager.getContact(contactId);
        assertEquals(contact, result);
        assertDeepEquals(contact, result);
    }

    @Test
    public void getAllContacts() {

        assertTrue(manager.getAll().isEmpty());

        Contact john = newContact("John", "+420 777 777 777");
        Contact george = newContact("George", "+420 543 438 190"); 

        manager.addContact(john);
        manager.addContact(george);

        List<Contact> expected = Arrays.asList(john,george);
        List<Contact> actual = manager.getAll();

        Collections.sort(actual,idComparator);
        Collections.sort(expected,idComparator);

        assertEquals(expected, actual);
        assertDeepEquals(expected, actual);
    }

    @Test
    public void addContactWithWrongAttributes() {

        try {
            manager.addContact(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        Contact contact = newContact("George", "+420 543 438 190"); 
        contact.setId(1l);
        try {
            manager.addContact(contact);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        contact = newContact(null, "+420 543 438 190"); 
        try {
            manager.addContact(contact);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        contact = newContact("George", null); 
        try {
            manager.addContact(contact);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

    }

    private static Contact newContact(String name, String phonenumber) {
        Contact contact = new Contact();
        contact.setName(name);
        contact.setPhoneNumber(phonenumber);
        return contact;
    }

    private void assertDeepEquals(List<Contact> expectedList, List<Contact> actualList) {
        for (int i = 0; i < expectedList.size(); i++) {
            Contact expected = expectedList.get(i);
            Contact actual = actualList.get(i);
            assertDeepEquals(expected, actual);
        }
    }

    private void assertDeepEquals(Contact expected, Contact actual) {
        assertEquals(expected.getId(), actual.getId());
        assertEquals(expected.getName(), actual.getName());
        assertEquals(expected.getPhoneNumber(), actual.getPhoneNumber());
    }

    private static Comparator<Contact> idComparator = new Comparator<Contact>() {

        @Override
        public int compare(Contact o1, Contact o2) {
            return Long.valueOf(o1.getId()).compareTo(Long.valueOf(o2.getId()));
        }
    };

}
